@@ -32,8 +32,15 @@ from django_smartbase_admin.engine.const import (
     CONFIG_NAME,
     TABLE_PARAMS_FULL_TEXT_SEARCH,
     TABLE_PARAMS_SELECTED_FILTER_TYPE,
+    FilterVersions,
+    ADVANCED_FILTER_DATA_NAME,
 )
 from django_smartbase_admin.services.views import SBAdminViewService
+from django_smartbase_admin.utils import import_with_injection
+
+QueryBuilderService = import_with_injection(
+    "django_smartbase_admin.actions.advanced_filters", "QueryBuilderService"
+)
 
 
 class SBAdminListAction(SBAdminAction):
@@ -51,20 +58,23 @@ class SBAdminListAction(SBAdminAction):
             self.all_params = json.loads(
                 request.request_data.request_get.get(BASE_PARAMS_NAME, "{}")
             )
+            source_data = request.request_data.request_get.get(BASE_PARAMS_NAME, "{}")
             if (
-                request.headers.get("X-TabulatorRequest", None) == "true"
-                and request.request_data.request_method == "POST"
+                request.request_data.request_method == "POST"
+                and request.headers.get("X-TabulatorRequest", None) == "true"
             ):
-                try:
-                    self.all_params = json.loads(request.body)
-                except json.JSONDecodeError:
-                    pass
+                source_data = request.body
+            try:
+                self.all_params = json.loads(source_data)
+            except json.JSONDecodeError:
+                pass
         else:
             self.all_params = all_params
         if not self.all_params:
             self.all_params = {}
         self.params = self.all_params.get(self.view.get_id(), {})
         self.filter_data = self.params.get(FILTER_DATA_NAME, {})
+        self.advanced_filter_data = self.params.get(ADVANCED_FILTER_DATA_NAME, {})
         self.table_params = self.params.get(TABLE_PARAMS_NAME, {})
         self.columns_data = self.params.get(COLUMNS_DATA_NAME, {})
         self.selection_data = self.params.get(SELECTION_DATA_NAME, {})
@@ -122,6 +132,11 @@ class SBAdminListAction(SBAdminAction):
             if field.field in values
         ]
 
+    def is_jquery_required(self, request):
+        return (
+            self.view.get_filters_version(request) == FilterVersions.FILTERS_VERSION_2
+        )
+
     def get_template_data(self):
         context_data = self.view.get_context_data(self.threadsafe_request)
         constants = {
@@ -141,6 +156,7 @@ class SBAdminListAction(SBAdminAction):
             "TABLE_PARAMS_FULL_TEXT_SEARCH": TABLE_PARAMS_FULL_TEXT_SEARCH,
             "TABLE_PARAMS_SELECTED_FILTER_TYPE": TABLE_PARAMS_SELECTED_FILTER_TYPE,
             "FILTER_DATA_NAME": FILTER_DATA_NAME,
+            "ADVANCED_FILTER_DATA_NAME": ADVANCED_FILTER_DATA_NAME,
             "BASE_PARAMS_NAME": BASE_PARAMS_NAME,
             "TABLE_PARAMS_PAGE_NAME": TABLE_PARAMS_PAGE_NAME,
             "TABLE_PARAMS_SORT_NAME": TABLE_PARAMS_SORT_NAME,
@@ -167,6 +183,13 @@ class SBAdminListAction(SBAdminAction):
                 "tabulator_definition": tabulator_definition,
                 "id_column_name": id_column_name,
                 "filters": self.get_filters(),
+                "is_jquery_required": self.is_jquery_required(self.threadsafe_request),
+                "advanced_filters_data": QueryBuilderService.get_advanced_filters_context_data(
+                    self
+                ),
+                "filters_template_name": self.view.get_filters_template_name(
+                    self.threadsafe_request
+                ),
                 "tabulator_header_template_name": self.view.get_tabulator_header_template_name(
                     self.threadsafe_request
                 ),
@@ -198,9 +221,11 @@ class SBAdminListAction(SBAdminAction):
         return order_by
 
     def get_filter_from_request(self):
-        return SBAdminViewService.get_filter_from_request(
+        base_filters = SBAdminViewService.get_filter_from_request(
             self.threadsafe_request, self.column_fields, self.filter_data
         )
+        advanced_filters = QueryBuilderService.get_filters_for_list_action(self)
+        return base_filters & advanced_filters
 
     def get_search_fields(self, request):
         search_fields_definition = self.view.get_search_fields(request)
@@ -218,6 +243,9 @@ class SBAdminListAction(SBAdminAction):
                 self.filter_data,
             ).keys()
         )
+        filter_fields.extend(
+            QueryBuilderService.get_filters_fields_for_list_action(self)
+        )
         if self.is_search_query():
             search_fields = self.get_search_fields(self.threadsafe_request)
             filter_fields.extend(search_fields)
