@@ -5,17 +5,26 @@ from ckeditor.fields import RichTextFormField
 from ckeditor_uploader.fields import RichTextUploadingFormField
 from django import forms
 from django.contrib import admin
+from django.contrib.admin.options import get_content_type_for_model
+from django.contrib.admin.utils import unquote
 from django.contrib.admin.widgets import AdminTextareaWidget
 from django.contrib.auth.forms import UsernameField, ReadOnlyPasswordHashWidget
 from django.contrib.postgres.forms import SimpleArrayField
-from django.core.exceptions import FieldDoesNotExist, ImproperlyConfigured
+from django.core.exceptions import (
+    FieldDoesNotExist,
+    ImproperlyConfigured,
+    PermissionDenied,
+)
 from django.db import models
 from django.forms import HiddenInput
 from django.forms.models import ModelFormMetaclass
 from django.template.loader import render_to_string
+from django.template.response import TemplateResponse
 from django.urls import reverse
 from django.utils.safestring import mark_safe
+from django.utils.text import capfirst
 from django.utils.translation import gettext_lazy as _
+from django.utils.translation import override as translation_override
 from django_admin_inline_paginator.admin import TabularInlinePaginated
 from filer.fields.image import AdminImageFormField
 from nested_admin.nested import (
@@ -57,7 +66,6 @@ from django_smartbase_admin.admin.widgets import (
     SBAdminSplitDateTimeWidget,
     SBAdminTimeWidget,
     SBAdminDateTimeWidget,
-    SBAdminAutocompleteWidget,
     SBAdminFileWidget,
     SBAdminToggleWidget,
     SBAdminNullBooleanSelectWidget,
@@ -66,6 +74,7 @@ from django_smartbase_admin.admin.widgets import (
     SBAdminPasswordInputWidget,
     SBAdminReadOnlyPasswordHashWidget,
     SBAdminHiddenWidget,
+    SBAdminCKEditorUploadingWidget,
 )
 from django_smartbase_admin.engine.admin_base_view import (
     SBAdminBaseListView,
@@ -101,7 +110,7 @@ class SBAdminFormFieldWidgetsMixin:
         forms.BooleanField: SBAdminToggleWidget,
         forms.SlugField: SBAdminTextInputWidget,
         RichTextFormField: SBAdminCKEditorWidget,
-        RichTextUploadingFormField: SBAdminCKEditorWidget,
+        RichTextUploadingFormField: SBAdminCKEditorUploadingWidget,
         forms.ChoiceField: SBAdminSelectWidget,
         forms.TypedChoiceField: SBAdminSelectWidget,
         forms.NullBooleanField: SBAdminNullBooleanSelectWidget,
@@ -118,6 +127,9 @@ class SBAdminFormFieldWidgetsMixin:
 
     def get_form_field_widget_class(self, form_field, db_field, request):
         default_widget_class = self.formfield_widgets.get(form_field.__class__)
+        if not hasattr(request, "request_data"):
+            # in case of login the view is not wrapped and we have no request_data present
+            return default_widget_class
         return request.request_data.configuration.get_form_field_widget_class(
             self, request, form_field, db_field, default_widget_class
         )
@@ -150,7 +162,14 @@ class SBAdminFormFieldWidgetsMixin:
         widget_attrs.pop(
             "class", None
         )  # remove origin classes to prevent override our custom widget class
-        form_field.widget = widget(form_field=form_field, attrs=widget_attrs)
+        kwargs = {}
+        if isinstance(form_field, RichTextFormField):
+            kwargs["config_name"] = getattr(db_field, "config_name", "default")
+            kwargs["external_plugin_resources"] = getattr(
+                db_field, "external_plugin_resources", []
+            )
+            kwargs["extra_plugins"] = getattr(db_field, "extra_plugins", [])
+        form_field.widget = widget(form_field=form_field, attrs=widget_attrs, **kwargs)
         return form_field
 
     def formfield_for_foreignkey(self, db_field, request, **kwargs):
@@ -193,6 +212,7 @@ class SBAdminFormFieldWidgetsMixin:
                 form_field_widget_instance = self.get_autocomplete_widget(
                     request,
                     form_field,
+                    db_field,
                     db_field.target_field.model,
                     multiselect=True,
                 )
@@ -225,6 +245,8 @@ class SBAdminBaseFormInit(SBAdminFormFieldWidgetsMixin, FormFieldsetMixin):
     view = None
 
     def __init__(self, *args, **kwargs):
+        self.view = kwargs.pop("view", self.view)
+        self.threadsafe_request = kwargs.pop("request", self.threadsafe_request)
         super().__init__(*args, **kwargs)
         for field in self.fields:
             if not hasattr(self.fields[field].widget, "init_widget_dynamic"):
@@ -416,13 +438,14 @@ class SBAdminInlineAndAdminCommon(SBAdminFormFieldWidgetsMixin):
         inlines = []
         for inline_class in inline_classes:
             inline = inline_class(self.model, self.admin_site)
+            if hasattr(inline, "init_inline_dynamic"):
+                inline.init_inline_dynamic(request, obj)
             if request:
                 if not inline.has_view_or_change_permission(request, obj):
                     continue
                 if not inline.has_add_permission(request, obj):
                     inline.max_num = 0
-            if hasattr(inline, "init_inline_dynamic"):
-                inline.init_inline_dynamic(request, obj)
+            if hasattr(inline, "init_view_dynamic"):
                 inline.init_view_dynamic(request, request.request_data)
             inlines.append(inline)
         return inlines
@@ -588,6 +611,12 @@ class SBAdmin(
     request_data = None
     menu_label = None
 
+    def save_formset(self, request, form, formset, change):
+        if not change and hasattr(formset, "inline_instance"):
+            # update inline_instance parent_instance on formset when creating new object
+            formset.inline_instance.parent_instance = form.instance
+        super().save_formset(request, form, formset, change)
+
     def get_sbadmin_list_filter(self, request):
         return self.sbadmin_list_filter or self.get_list_filter(request)
 
@@ -732,6 +761,80 @@ class SBAdmin(
     def changelist_view(self, request, extra_context=None):
         return self.action_list(request, extra_context=extra_context)
 
+    def _get_changed_field_labels_from_form(form, changed_data):
+        changed_field_labels = []
+        for field_name in changed_data:
+            try:
+                verbose_field_name = form.fields[field_name].label or field_name
+            except KeyError:
+                verbose_field_name = field_name
+            changed_field_labels.append(str(verbose_field_name))
+        return changed_field_labels
+
+    def history_view(self, request, object_id, extra_context=None):
+        try:
+            "The 'history' admin view for this model."
+            from django.contrib.admin.models import LogEntry
+            from django.contrib.admin.views.main import PAGE_VAR
+
+            # First check if the user can see this history.
+            model = self.model
+            obj = self.get_object(request, unquote(object_id))
+            if obj is None:
+                return self._get_obj_does_not_exist_redirect(
+                    request, model._meta, object_id
+                )
+
+            if not self.has_view_or_change_permission(request, obj):
+                raise PermissionDenied
+
+            # Then get the history for this object.
+            app_label = self.opts.app_label
+            action_list = (
+                LogEntry.objects.filter(
+                    object_id=unquote(object_id),
+                    content_type=get_content_type_for_model(model),
+                )
+                .select_related()
+                .order_by("-action_time")
+            )
+
+            paginator = self.get_paginator(request, action_list, 100)
+            page_number = request.GET.get(PAGE_VAR, 1)
+            page_obj = paginator.get_page(page_number)
+            page_range = paginator.get_elided_page_range(page_obj.number)
+
+            context = {
+                **self.admin_site.each_context(request),
+                "title": _("Change history: %s") % obj,
+                "subtitle": None,
+                "action_list": page_obj,
+                "page_range": page_range,
+                "page_var": PAGE_VAR,
+                "pagination_required": paginator.count > 100,
+                "module_name": str(capfirst(self.opts.verbose_name_plural)),
+                "object": obj,
+                "opts": self.opts,
+                "preserved_filters": self.get_preserved_filters(request),
+                **(extra_context or {}),
+            }
+
+            request.current_app = self.admin_site.name
+
+            return TemplateResponse(
+                request,
+                self.object_history_template
+                or [
+                    "admin/%s/%s/object_history.html"
+                    % (app_label, self.opts.model_name),
+                    "admin/%s/object_history.html" % app_label,
+                    "admin/object_history.html",
+                ],
+                context,
+            )
+        except Exception as e:
+            return super().history_view(request, object_id, extra_context)
+
 
 class SBAdminInline(
     SBAdminInlineAndAdminCommon, SBAdminBaseQuerysetMixin, SBAdminBaseView
@@ -741,6 +844,17 @@ class SBAdminInline(
     sbadmin_sortable_field_options = ["order_by"]
     sbadmin_inline_list_actions = None
     extra = 0
+    ordering = None
+
+    def get_ordering(self, request):
+        """
+        Hook for specifying field ordering.
+        """
+        return self.ordering or ("-id",)
+
+    def get_queryset(self, request=None):
+        qs = super().get_queryset(request)
+        return qs.order_by(*self.get_ordering(request))
 
     def get_sbadmin_inline_list_actions(self):
         return [*(self.sbadmin_inline_list_actions or [])]
