@@ -2,6 +2,7 @@ import json
 import urllib.parse
 from collections import defaultdict
 
+from django.contrib import messages
 from django.contrib.admin.actions import delete_selected
 from django.core.exceptions import PermissionDenied
 from django.db.models import F
@@ -24,6 +25,9 @@ from django_smartbase_admin.engine.const import (
     GLOBAL_FILTER_ALIAS_WIDGET_ID,
     OVERRIDE_CONTENT_OF_NOTIFICATION,
     FilterVersions,
+    BASE_PARAMS_NAME,
+    TABLE_RELOAD_DATA_EVENT_NAME,
+    TABLE_UPDATE_ROW_DATA_EVENT_NAME,
 )
 from django_smartbase_admin.services.views import SBAdminViewService
 from django_smartbase_admin.services.xlsx_export import (
@@ -31,7 +35,7 @@ from django_smartbase_admin.services.xlsx_export import (
     SBAdminXLSXOptions,
     SBAdminXLSXFormat,
 )
-from django_smartbase_admin.utils import is_htmx_request
+from django_smartbase_admin.utils import is_htmx_request, render_notifications
 
 
 class SBAdminBaseView(object):
@@ -172,6 +176,8 @@ class SBAdminBaseView(object):
                     "MULTISELECT_FILTER_MAX_CHOICES_SHOWN": MULTISELECT_FILTER_MAX_CHOICES_SHOWN,
                     "AUTOCOMPLETE_PAGE_SIZE": AUTOCOMPLETE_PAGE_SIZE,
                     "GLOBAL_FILTER_ALIAS_WIDGET_ID": GLOBAL_FILTER_ALIAS_WIDGET_ID,
+                    "TABLE_RELOAD_DATA_EVENT_NAME": TABLE_RELOAD_DATA_EVENT_NAME,
+                    "TABLE_UPDATE_ROW_DATA_EVENT_NAME": TABLE_UPDATE_ROW_DATA_EVENT_NAME,
                 }
             ),
         }
@@ -287,7 +293,8 @@ class SBAdminBaseListView(SBAdminBaseView):
         current_row_id = json.loads(request.POST.get("currentRowId", ""))
         column_field_name = request.POST.get("columnFieldName", "")
         cell_value = request.POST.get("cellValue", "")
-        return JsonResponse({"message": "Not Implemented"})
+        messages.add_message(request, messages.ERROR, "Not Implemented")
+        return HttpResponse(status=200, content=render_notifications(request))
 
     def init_view_dynamic(self, request, request_data=None, **kwargs):
         super().init_view_dynamic(request, request_data, **kwargs)
@@ -310,6 +317,12 @@ class SBAdminBaseListView(SBAdminBaseView):
             for field in self.get_sbamin_list_display(request)
         ]
 
+    def get_search_fields(self, request):
+        if hasattr(super(SBAdminBaseListView, self), "get_search_fields"):
+            return super().get_search_fields(request)
+        else:
+            return []
+
     def get_list_ordering(self):
         return self.ordering or []
 
@@ -337,6 +350,7 @@ class SBAdminBaseListView(SBAdminBaseView):
         view_id = self.get_id()
         tabulator_definition = {
             "viewId": view_id,
+            "advancedFilterId": f"{view_id}" + "-advanced-filter",
             "filterFormId": f"{view_id}" + "-filter-form",
             "columnWidgetId": f"{view_id}" + "-column-widget",
             "paginationWidgetId": f"{view_id}" + "-pagination-widget",
@@ -362,12 +376,11 @@ class SBAdminBaseListView(SBAdminBaseView):
                 "filterModule",
                 "tableParamsModule",
                 "detailViewModule",
-                "fullTextSearchModule",
-                "filterOptionsModule",
             ],
             "tabulatorOptions": {
                 "renderVertical": "basic",
                 "persistence": False,
+                "layoutColumnsOnNewData": True,
                 "height": "100%",
                 "ajaxContentType": "json",
                 "ajaxConfig": {
@@ -383,6 +396,14 @@ class SBAdminBaseListView(SBAdminBaseView):
                 "selectable": "highlight",
             },
         }
+        if self.get_filters_version(request) == FilterVersions.FILTERS_VERSION_2:
+            tabulator_definition["modules"].extend(
+                [
+                    "advancedFilterModule",
+                    "fullTextSearchModule",
+                    "headerTabsModule",
+                ]
+            )
         return tabulator_definition
 
     def _get_sbadmin_list_actions(self):
@@ -457,6 +478,21 @@ class SBAdminBaseListView(SBAdminBaseView):
 
     def action_bulk_delete(self, request, modifier):
         action = SBAdminListAction(self, request)
+        if (
+            request.request_data.request_method == "POST"
+            and request.headers.get("X-TabulatorRequest", None) == "true"
+        ):
+            return redirect(
+                self.get_action_url("action_bulk_delete")
+                + "?"
+                + urllib.parse.urlencode(
+                    {BASE_PARAMS_NAME: json.dumps(action.all_params)}
+                )
+            )
+        if not action.selection_data:
+            # don't run with no selection data as it will result in delete of all records
+            messages.error(request, _("No selection made."))
+            return redirect(self.get_menu_view_url(request))
         additional_filter = action.get_selection_queryset()
         response = delete_selected(
             self, request, self.get_queryset(request).filter(additional_filter)
@@ -473,33 +509,48 @@ class SBAdminBaseListView(SBAdminBaseView):
         name = config_name or request.POST.get(CONFIG_NAME, None)
         if name:
             name = urllib.parse.unquote(name)
+        updated_configuration = None
         if request.request_data.request_method == "POST":
             if name:
-                SBAdminListViewConfiguration.objects.update_or_create(
-                    name=name,
-                    user_id=request.request_data.user.id,
-                    defaults={
-                        "url_params": request.request_data.request_post.get(
-                            URL_PARAMS_NAME
-                        ),
-                        "view": self.get_id(),
-                        "action": None,
-                        "modifier": None,
-                    },
+                updated_configuration, created = (
+                    SBAdminListViewConfiguration.objects.update_or_create(
+                        name=name,
+                        user_id=request.request_data.user.id,
+                        defaults={
+                            "url_params": request.request_data.request_post.get(
+                                URL_PARAMS_NAME
+                            ),
+                            "view": self.get_id(),
+                            "action": None,
+                            "modifier": None,
+                        },
+                    )
                 )
         if request.request_data.request_method == "DELETE":
             if name:
                 SBAdminListViewConfiguration.objects.by_user_id(
                     request.request_data.user.id
                 ).by_name(name).by_view_action_modifier(view=self.get_id()).delete()
-        response = redirect(request.request_data.request_meta.get("HTTP_REFERER", ""))
+
+        redirect_to = self.get_redirect_url_from_request(request, updated_configuration)
+
+        response = redirect(redirect_to)
         if is_htmx_request(request.request_data.request_meta):
             response = HttpResponse()
-            response["HX-Redirect"] = request.request_data.request_meta.get(
-                "HTTP_REFERER", ""
-            )
+            response["HX-Redirect"] = redirect_to
         return response
 
+    def get_redirect_url_from_request(self, request, updated_configuration=None):
+        referer = request.request_data.request_meta.get("HTTP_REFERER", "")
+        url = urllib.parse.urlparse(referer)
+        query = dict(urllib.parse.parse_qsl(url.query))
+        query.update({"tabCreated": True})
+        if updated_configuration:
+            query.update({"selectedView": updated_configuration.pk})
+        url = url._replace(query=urllib.parse.urlencode(query))
+        redirect_to = urllib.parse.urlunparse(url)
+        return redirect_to
+
     def action_list(
         self,
         request,
@@ -634,6 +685,14 @@ class SBAdminBaseListView(SBAdminBaseView):
             self.filters_version or request.request_data.configuration.filters_version
         )
 
+    def get_filters_template_name(self, request):
+        filters_version = self.get_filters_version(request)
+        if filters_version is FilterVersions.FILTERS_VERSION_2:
+            return "sb_admin/components/filters_v2.html"
+        else:
+            # default
+            return "sb_admin/components/filters.html"
+
     def get_tabulator_header_template_name(self, request):
         filters_version = self.get_filters_version(request)
         if filters_version is FilterVersions.FILTERS_VERSION_2:
