@@ -2,11 +2,17 @@ import json
 from datetime import datetime, timedelta
 
 from django.core.exceptions import ImproperlyConfigured
-from django.db.models import Q, fields, FilteredRelation
+from django.db.models import Q, fields, FilteredRelation, Count
 from django.http import JsonResponse
 from django.utils import timezone
 from django.utils.translation import gettext_lazy as _
 
+from django_smartbase_admin.actions.advanced_filters import (
+    AllOperators,
+    STRING_ATTRIBUTES,
+    NUMBER_ATTRIBUTES,
+    DATE_ATTRIBUTES,
+)
 from django_smartbase_admin.engine.admin_view import SBAdminView
 from django_smartbase_admin.engine.const import (
     AUTOCOMPLETE_SEARCH_NAME,
@@ -50,6 +56,7 @@ class SBAdminFilterWidget(JSONSerializableMixin):
     default_value = None
     default_label = None
     filter_query_lambda = None
+    exclude_null_operators = False
 
     def __init__(
         self,
@@ -57,6 +64,7 @@ class SBAdminFilterWidget(JSONSerializableMixin):
         default_value=None,
         default_label=None,
         filter_query_lambda=None,
+        exclude_null_operators=None,
         **kwargs,
     ) -> None:
         super().__init__()
@@ -64,6 +72,9 @@ class SBAdminFilterWidget(JSONSerializableMixin):
         self.default_value = self.default_value or default_value
         self.default_label = self.default_label or default_label
         self.filter_query_lambda = filter_query_lambda or self.filter_query_lambda
+        self.exclude_null_operators = (
+            exclude_null_operators or self.exclude_null_operators
+        )
 
     def init_filter_widget_static(self, field, view, configuration):
         self.field = field
@@ -89,6 +100,11 @@ class SBAdminFilterWidget(JSONSerializableMixin):
     def get_base_filter_query_for_parsed_value(self, request, parsed_value):
         return Q(**{f"{self.field.filter_field}__icontains": parsed_value})
 
+    def get_advanced_filter_query_for_parsed_value(
+        self, request, parsed_value, original_query, rule
+    ):
+        return original_query
+
     def to_json(self):
         return {"input_id": self.input_id}
 
@@ -117,10 +133,21 @@ class SBAdminFilterWidget(JSONSerializableMixin):
         context["filter_widget"] = self
         return context
 
+    def get_advanced_filter_operators(self):
+        return [
+            AllOperators.EQUAL,
+            AllOperators.NOT_EQUAL,
+            AllOperators.IS_NULL,
+            AllOperators.IS_NOT_NULL,
+        ]
+
 
 class StringFilterWidget(SBAdminFilterWidget):
     template_name = "sb_admin/filter_widgets/string_field.html"
 
+    def get_advanced_filter_operators(self):
+        return STRING_ATTRIBUTES
+
     @classmethod
     def is_used_for_model_field_type(cls, model_field):
         return (
@@ -134,6 +161,17 @@ class StringFilterWidget(SBAdminFilterWidget):
 class BooleanFilterWidget(SBAdminFilterWidget):
     template_name = "sb_admin/filter_widgets/boolean_field.html"
 
+    def parse_value_from_input(self, request, filter_value):
+        input_value = super().parse_value_from_input(request, filter_value)
+        try:
+            input_value = json.loads(input_value)
+        except:
+            pass
+
+        if input_value is None:
+            return None
+        return input_value
+
     @classmethod
     def is_used_for_model_field_type(cls, model_field):
         return isinstance(model_field, fields.BooleanField)
@@ -186,20 +224,42 @@ class MultipleChoiceFilterWidget(AutocompleteParseMixin, ChoiceFilterWidget):
     def get_base_filter_query_for_parsed_value(self, request, filter_value):
         return Q(**{f"{self.field.filter_field}__in": filter_value})
 
+    def get_advanced_filter_operators(self):
+        return [
+            AllOperators.IN,
+            AllOperators.NOT_IN,
+            AllOperators.IS_NULL,
+            AllOperators.IS_NOT_NULL,
+        ]
 
-class NumberRangeFilterWidget(SBAdminFilterWidget):
+
+class NumberRangeFilterWidget(AutocompleteParseMixin, SBAdminFilterWidget):
     template_name = "sb_admin/filter_widgets/number_range_field.html"
 
+    def parse_value_from_input(self, request, filter_value):
+        filter_value = super().parse_value_from_input(request, filter_value)
+        try:
+            from_value = float(filter_value.get("from", {}).get("value", None))
+        except (ValueError, TypeError):
+            from_value = None
+        try:
+            to_value = float(filter_value.get("to", {}).get("value", None))
+        except (ValueError, TypeError):
+            to_value = None
+
+        return [from_value, to_value]
+
     def get_base_filter_query_for_parsed_value(self, request, filter_value):
-        from_value = filter_value.get("from", {}).get("value", None)
-        to_value = filter_value.get("to", {}).get("value", None)
         result = Q()
-        if from_value:
-            result &= Q(**{f"{self.field.filter_field}__gte": from_value})
-        if to_value:
-            result &= Q(**{f"{self.field.filter_field}__lte": to_value})
+        if filter_value[0] is not None:
+            result &= Q(**{f"{self.field.filter_field}__gte": filter_value[0]})
+        if filter_value[1] is not None:
+            result &= Q(**{f"{self.field.filter_field}__lte": filter_value[1]})
         return result
 
+    def get_advanced_filter_operators(self):
+        return NUMBER_ATTRIBUTES
+
 
 class DateFilterWidget(SBAdminFilterWidget):
     DATE_RANGE_SPLIT = " - "
@@ -224,6 +284,7 @@ class DateFilterWidget(SBAdminFilterWidget):
             "label": _("Last 12 months"),
         },
     ]
+    shortcuts_dict = {AllOperators.IN_THE_LAST.value: shortcuts}
     default_value_shortcut_index = None
 
     def __init__(
@@ -243,24 +304,37 @@ class DateFilterWidget(SBAdminFilterWidget):
             else self.default_value_shortcut_index
         )
 
+    def get_advanced_filter_operators(self):
+        return DATE_ATTRIBUTES
+
+    def process_shortcut(self, shortcut, now):
+        return {
+            "label": shortcut["label"],
+            "value": [
+                now + timedelta(days=shortcut["value"][0]),
+                now + timedelta(days=shortcut["value"][1]),
+            ],
+        }
+
     def get_shortcuts(self):
         now = timezone.now()
         shortcuts = []
         for shortcut in self.shortcuts:
-            shortcuts.append(
-                {
-                    "label": shortcut["label"],
-                    "value": [
-                        now + timedelta(days=shortcut["value"][0]),
-                        now + timedelta(days=shortcut["value"][1]),
-                    ],
-                }
-            )
+            shortcuts.append(self.process_shortcut(shortcut, now))
+        return shortcuts
+
+    def get_shortcuts_dict(self):
+        now = timezone.now()
+        shortcuts = {}
+        for key, shortcuts_group in self.shortcuts_dict.items():
+            shortcuts[key] = []
+            for shortcut in shortcuts_group:
+                shortcuts[key].append(self.process_shortcut(shortcut, now))
         return shortcuts
 
     def get_default_value(self):
         if self.default_value_shortcut_index is not None:
-            selected_shortcut_value = self.get_shortcuts()[
+            selected_shortcut_value = next(iter(self.get_shortcuts()), [])[
                 self.default_value_shortcut_index
             ]["value"]
             return self.get_value_from_date_or_range(selected_shortcut_value)
@@ -269,7 +343,6 @@ class DateFilterWidget(SBAdminFilterWidget):
     def get_data(self):
         return json.dumps(
             {
-                "shortcuts": self.get_shortcuts(),
                 "flatpickrOptions": {
                     "locale": {
                         "rangeSeparator": self.DATE_RANGE_SPLIT,
@@ -279,26 +352,41 @@ class DateFilterWidget(SBAdminFilterWidget):
             cls=SBAdminJSONEncoder,
         )
 
+    def get_shortcuts_data(self):
+        return json.dumps(
+            self.get_shortcuts(),
+            cls=SBAdminJSONEncoder,
+        )
+
+    def get_shortcuts_dict_data(self):
+        # used for advanced filters with different calendar operators "in the last", "in the next", etc.
+        return json.dumps(
+            self.get_shortcuts_dict(),
+            cls=SBAdminJSONEncoder,
+        )
+
     @classmethod
     def is_used_for_model_field_type(cls, model_field):
         return isinstance(model_field, fields.DateField)
 
     @classmethod
-    def get_date_or_range_from_value(cls, filter_value):
+    def get_range_from_value(cls, filter_value):
         """
-        Get single date or date-range from string filter value
+        Get date-range from string filter value
 
-        :returns: date or array or dates, is_range
+        :returns: array, is_range
         """
+        if filter_value is None:
+            return [None, None]
         date_format = cls.DATE_FORMAT
         date_range = filter_value.split(cls.DATE_RANGE_SPLIT)
         if len(date_range) == 2:
             date_from = datetime.strptime(date_range[0], date_format)
             date_to = datetime.strptime(date_range[1], date_format)
-            return [date_from, date_to], True
+            return [date_from, date_to]
         else:
             date_value = datetime.strptime(filter_value, date_format)
-            return [date_value, date_value], True
+            return [date_value, date_value]
 
     @classmethod
     def get_value_from_date_or_range(cls, date_or_range):
@@ -308,21 +396,19 @@ class DateFilterWidget(SBAdminFilterWidget):
         date_to = datetime.strftime(date_or_range[1], cls.DATE_FORMAT)
         return f"{date_from}{cls.DATE_RANGE_SPLIT}{date_to}"
 
+    def parse_value_from_input(self, request, filter_value):
+        return self.get_range_from_value(filter_value)
+
     def get_base_filter_query_for_parsed_value(self, request, filter_value):
-        date_or_range, is_range = self.get_date_or_range_from_value(filter_value)
-        if is_range:
-            date_from = date_or_range[0]
-            # add one day to include all 'to' date hours, this is needed instead of casting datetime to date in DB
-            date_to = date_or_range[1] + timedelta(days=1)
-            return Q(
-                **{
-                    f"{self.field.filter_field}__gte": date_from,
-                    f"{self.field.filter_field}__lte": date_to,
-                }
-            )
-        else:
-            date_value = date_or_range
-            return Q(**{f"{self.field.filter_field}": date_value})
+        date_from = filter_value[0]
+        # add one day to include all 'to' date hours, this is needed instead of casting datetime to date in DB
+        date_to = filter_value[1] + timedelta(days=1)
+        return Q(
+            **{
+                f"{self.field.filter_field}__gte": date_from,
+                f"{self.field.filter_field}__lte": date_to,
+            }
+        )
 
 
 class AutocompleteFilterWidget(
@@ -339,6 +425,7 @@ class AutocompleteFilterWidget(
     value_lambda = None
     allow_add = False
     hide_clear_button = False
+    search_query_lambda = None
 
     def get_field_name(self):
         return self.field.name
@@ -360,6 +447,7 @@ class AutocompleteFilterWidget(
         forward=None,
         allow_add=None,
         hide_clear_button=None,
+        search_query_lambda=None,
         **kwargs,
     ) -> None:
         super().__init__(template_name, default_value, **kwargs)
@@ -367,6 +455,7 @@ class AutocompleteFilterWidget(
         self.value_field = value_field or self.value_field
         self.filter_query_lambda = filter_query_lambda or self.filter_query_lambda
         self.filter_search_lambda = filter_search_lambda or self.filter_search_lambda
+        self.search_query_lambda = search_query_lambda or self.search_query_lambda
         self.label_lambda = label_lambda or self.label_lambda
         self.value_lambda = value_lambda or self.value_lambda
         self.multiselect = multiselect if multiselect is not None else self.multiselect
@@ -496,13 +585,24 @@ class AutocompleteFilterWidget(
         to_item = (page_num) * AUTOCOMPLETE_PAGE_SIZE
         qs = self.get_queryset(request)
         qs = self.filter_search_queryset(request, qs, search_term, forward_data)
-        qs = self.get_default_search_query(
-            request,
-            qs,
-            self.model,
-            search_term,
-            SBAdminTranslationsService.get_main_lang_code(),
-        )[from_item:to_item]
+        if self.search_query_lambda:
+            qs = self.search_query_lambda(
+                request,
+                qs,
+                self.model,
+                search_term,
+                SBAdminTranslationsService.get_main_lang_code(),
+            )
+        else:
+            qs = self.get_default_search_query(
+                request,
+                qs,
+                self.model,
+                search_term,
+                SBAdminTranslationsService.get_main_lang_code(),
+            )
+
+        qs = qs[from_item:to_item]
         result = []
         for item in qs:
             result.append(
@@ -527,5 +627,56 @@ class AutocompleteFilterWidget(
         context = super().get_context(name, value, attrs)
         context["filter_widget"] = self
         context["widget"]["type"] = "hidden"
-        context["widget"]["attrs"]["id"] = self.input_id
+        context["widget"]["attrs"]["id"] = (
+            self.input_id or context["widget"]["attrs"]["id"]
+        )
         return context
+
+    def get_advanced_filter_operators(self):
+        return [
+            AllOperators.IN,
+            AllOperators.NOT_IN,
+            AllOperators.IS_NULL,
+            AllOperators.IS_NOT_NULL,
+        ]
+
+
+class FromValuesAutocompleteWidget(AutocompleteFilterWidget):
+    def init_filter_widget_static(self, field, view, configuration):
+        self.model = field.view.model
+        super().init_filter_widget_static(field, view, configuration)
+
+    def get_queryset(self, request=None):
+        qs = super().get_queryset(request)
+        qs = qs.annotate(**self.field.get_field_annotates(values=[]))
+        return qs
+
+    def search(self, request, post_data):
+        search_term = post_data.get(AUTOCOMPLETE_SEARCH_NAME)
+        forward_data = json.loads(post_data.get(AUTOCOMPLETE_FORWARD_NAME, "{}"))
+        page_num = int(post_data.get(AUTOCOMPLETE_PAGE_NUM, 1))
+        from_item = (page_num - 1) * AUTOCOMPLETE_PAGE_SIZE
+        to_item = (page_num) * AUTOCOMPLETE_PAGE_SIZE
+        qs = self.get_queryset(request)
+        qs = self.filter_search_queryset(request, qs, search_term, forward_data)
+        qs = (
+            qs.filter(**{f"{self.field.name}__icontains": search_term})
+            .values(self.field.name)
+            .annotate(remove_duplicates_count=Count(self.field.name))
+            .order_by(self.field.name)[from_item:to_item]
+        )
+        result = []
+        for item in qs:
+            result.append(
+                {
+                    "value": self.get_value(request, item),
+                    "label": self.get_label(request, item),
+                }
+            )
+        return result
+
+    def get_value(self, request, item):
+        return item.get(self.field.name)
+
+    def get_label(self, request, item):
+        return item.get(self.field.name)
